import React, { useEffect, useState } from "react"
import { useDispatch, useSelector } from "react-redux"
import { AppDispatch, RootState } from "../stores/store"
import LoadingBox from "../components/loading/LoadingBox"
import { log } from "../logging/logger"
import { DrawerScreenProps } from "@react-navigation/drawer"
import { RootDrawerParamList } from "./Navigation"
import { Box, CloseIcon, Select, useToast, Button, Flex, Text, Center, VStack, ScrollView } from "native-base"
import { ErrorToast } from "../components/toast/ErrorToast"
import { Floor, Place, Room } from "../types/plan"
import { getFloorList, getPlanFloorImage, getPlanInventories, getPlanItems } from "../stores/actions/planThunks"
import { login } from "../stores/actions/userThunks"
import { getPlanItemsRequest } from "../api/planservice"
import ItemPreview from "../components/listView/ItemPreview"
import PlanView from "../components/plan/CastlePlanView"
import { BASE_API_URL } from "../api/constants"
import ListView from "../components/listView/ListView"
import SearchForm from "../components/search/SearchForm"
import { loadItemsByInventory } from "../stores/actions/listViewThunks"
import CastlePlanView from "../components/plan/CastlePlanView"

const PlanViewPage = ({ route, navigation }: DrawerScreenProps<RootDrawerParamList, 'Plan'>) => {

  const dispatch = useDispatch<AppDispatch>()

  // Route: set selected room / place from 
  useEffect(() => {
    console.log("dispatching get floor list")
    // dispatch(login({username: "Viktorie", password: "Golem123."}))
    dispatch(getFloorList())
  }, [])

  const DEFAULT_FLOOR = "first_floor"

  const [selectedFloor, setSelectedFloor] = useState<Floor | undefined>(undefined)
  const [selectedRoom, setSelectedRoom] = useState<Room | undefined>(undefined)
  const [selectedPlace, setSelectedPlace] = useState<Place | undefined>(undefined)

  const { floorListLoading, floorList, totalItemsCount, lastError } = useSelector((state: RootState) => state.planViewState)

  const inventories = useSelector((state: RootState) => state.planViewState.itemInventories)
  const inventoriesLoading = useSelector((state: RootState) => state.planViewState.itemInventoriesLoading)

  const numberOfResults = useSelector((state: RootState) => state.planViewState.totalItemsCount)

  const data = useSelector((state: RootState) => state.planViewState.itemList)
  const dataLoading = useSelector((state: RootState) => state.planViewState.itemListLoading)
  const pagination = useSelector((state: RootState) => state.planViewState.itemListPagination)

  const floorMapImage = useSelector((state: RootState) => state.planViewState.mapImage)
  const floorMapImageLoading = useSelector((state: RootState) => state.planViewState.mapImageLoading)

  const toast = useToast();

  useEffect(() => {
    if (floorList && !selectedFloor) {

      if (route.params && route.params.roomId) {

        selectRoom(route.params.roomId)

        if (route.params.placeId) {
          selectPlace(route.params.placeId)
        }

        // if (!itemListLoading) {
        //   searchSubmit(currentPage, route.params.roomId, route.params.placeId)
        // }
      }
      else {
        selectFloor(DEFAULT_FLOOR)
      }
    }
  }, [floorList, route.params.roomId, route.params.placeId])

  useEffect(() => {
    if (floorList) {
      if (!inventoriesLoading) {
        dispatch(getPlanInventories({ room: selectedRoom?.id ?? 0, place: selectedPlace?.id }))
      }
    }
  }, [selectedPlace, selectedRoom])

  useEffect(() => {
    if (selectedFloor) {
      console.log("Dispatching getFloorImage " + selectedFloor.id)
      dispatch(getPlanFloorImage(selectedFloor.id))
    }
  }, [selectedFloor])


  useEffect(() => {
    if (lastError) {
      toast.closeAll()
      toast.show({
        render: ({
          id
        }) => {
          return <ErrorToast headerText={"Error"} text={lastError} onClose={() => toast.close(id)} />;
        },
        duration: 3000
      });
    }
  }, [lastError])


  const searchSubmit = (roomId: number, placeId?: number) => {
    log.debug("PlanView", "searchSubmit")

    console.log("searching " + roomId + ", " + placeId)

    dispatch(getPlanInventories({ room: roomId, place: placeId }))
  }

  const selectFloor = (floorId: string) => {
    if (floorId == selectedFloor?.id) {
      return;
    }

    for (let i = 0; i < floorList.length; i++) {
      let floor = floorList[i]
      if (floor.id == floorId) {
        setSelectedFloor(floor)
        setSelectedRoom(undefined)
        setSelectedPlace(undefined)
        return;
      }
    }
  }

  const selectRoom = (roomId: number) => {
    if (roomId == selectedRoom?.id) {
      return;
    }

    for (let i = 0; i < floorList.length; i++) {
      let floor = floorList[i]

      for (let j = 0; j < floor.roomList.length; j++) {
        let room = floor.roomList[j]
        if (room.id == roomId) {
          setSelectedFloor(floor)
          setSelectedRoom(room)
          setSelectedPlace(undefined)
          return;
        }
      }
    }
  }

  const selectPlace = (placeId: number) => {
    if (placeId == selectedPlace?.id) {
      return;
    }

    for (let i = 0; selectedRoom?.places && i < selectedRoom?.places?.length; i++) {
      let place = selectedRoom?.places[i]
      if (place.id == placeId) {
        setSelectedPlace(place)
        return;
      }
    }
  }


  const clearForm = () => {
    log.debug("PlanView", "clearForm")
    setSelectedPlace(undefined)
    setSelectedRoom(undefined)
    selectFloor(DEFAULT_FLOOR)
  }

  const onBackPressed = () => {
    log.debug("back pressed")
    navigation.goBack();
  }

  return (
    <Center m={2} mr={0} mb={6} flex={1}>

      <ScrollView flex={1} w={"100%"} >
        <VStack space={1} mr={4}>

          {/* Selection */}

          {
            floorListLoading ?
              <LoadingBox text="Floor list loading"></LoadingBox>
              :
              <>
                {/* Floor */}
                <Select
                  mt={2.5}
                  ml={2.5}
                  mr={2.5}

                  placeholder={"Floor"}
                  selectedValue={selectedFloor?.id}
                  onValueChange={selectFloor}
                  variant="rounded"
                >
                  {floorList.map((floor, index) => {
                    return (
                      <Select.Item value={floor.id} label={floor.label} />
                    );
                  })}
                </Select>

                {/* Room */}
                {selectedFloor && selectedFloor.roomList && selectedFloor.roomList.length > 0 &&
                  < Select
                    mt={2.5}
                    ml={2.5}
                    mr={2.5}

                    placeholder={"Room"}
                    selectedValue={selectedRoom?.id.toString()}
                    onValueChange={(value: string) => selectRoom(parseInt(value))}
                    variant="rounded"
                  >
                    {selectedFloor?.roomList?.map((room, index) => {
                      if (room.room_list) {
                        return (
                          <Select.Item value={room.id.toString()} label={room.label} />
                        );
                      }
                    })}
                  </Select>
                }

                {/* Place */}
                {selectedRoom && selectedRoom.places && selectedRoom.places.length > 0 &&
                  <Select
                    mt={2.5}
                    ml={2.5}
                    mr={2.5}

                    placeholder={"Place"}
                    selectedValue={selectedPlace?.id.toString()}
                    onValueChange={(value: string) => selectPlace(parseInt(value))}
                    variant="rounded"
                  >
                    {selectedRoom?.places?.map((place, index) => {
                      return (
                        <Select.Item value={place.id.toString()} label={place.label} />
                      );
                    })}
                  </Select>
                }

                {/* Filter */}
                <Flex direction="row" alignItems="center" justify="flex-end"
                  mt={2.5}
                  ml={2.5}
                  mr={3}
                >
                  <Button
                    endIcon={
                      <CloseIcon size="xs" />
                    }
                    onPress={clearForm}
                    variant="outline"
                    color="primary.500"
                    borderColor="primary.100"
                    mr={2}
                    borderRadius={15}
                    size={"md"}
                  >
                    Reset
                  </Button>
                  <Button
                    onPress={() => {
                      if (selectedRoom && !selectedPlace) {
                        searchSubmit(selectedRoom.id)
                      }
                      else if (selectedRoom && selectedPlace) {
                        searchSubmit(selectedRoom.id, selectedPlace.id)
                      }

                    }}
                    colorScheme="primary"
                    background={"primary.100"}
                    variant="solid"
                    borderRadius={15}
                    size={"md"}
                  >
                    Search
                  </Button>
                </Flex>
              </>
          }

          {/* Plan */}
          {/* temporary prototype */}

          {selectedFloor && !floorListLoading && (
            floorMapImageLoading ?
              <LoadingBox text="Loading castle plan" />
              :
              floorMapImage &&
              <Box mt={2.5} ml={2.5}>
                <CastlePlanView mapImage={floorMapImage} selectedRoom={selectedRoom} />
              </Box>
          )
          }

          {/* Item List */}
          <>
            {inventoriesLoading ?
              <LoadingBox text="Loading available inventories..." />
              :
              <Box mt={2.5} ml={2.5}>
                <ListView

                  navigation={navigation}
                  hideHeaderText

                  inventories={inventories}
                  numOfResults={numberOfResults}

                  data={data}
                  loadedPages={pagination ?? {}}
                  inventoriesDataLoading={dataLoading ?? {}}
                  handleLoadMoreItems={(inventoryName: string, cursor: number) => {
                    dispatch(getPlanItems({ cursor: cursor, inventory: inventoryName, room: selectedRoom?.id ?? 0, place: selectedPlace?.id }))
                  }
                  }
                />
              </Box>
            }
          </>

        </VStack>
      </ScrollView>
    </Center>


  );
}

export default PlanViewPage;
